/* ntris -- a product of Sadistic Software. Copyright 1990, Andrew Plotkin. */

#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/time.h>
#include <curses.h>
#include <errno.h>
#include "ntris.h"

extern char *sprintf();
extern long random();
long game_speed;

init_game() /* play a set of games */
{
    struct timeval tp;
    char fb[LINELEN];

    (void)gettimeofday(&tp, (struct timezone *)NULL);
    srandom((int)tp.tv_sec);

    if ((signal (SIGHUP, (int(*)())signal_end) == BADSIG) ||
	 (signal (SIGINT, (int(*)())signal_end) == BADSIG) ||
	 (signal (SIGTERM, (int(*)())signal_end) == BADSIG))
	die (LE_SIGNAL, "couldn't set up signal-handling");

    setup_curses ();

    if (COLS < 2*game_width+15 || LINES < game_depth+2) {
	sprintf(fb, "this piece set requires a %d by %d character window.", 2*game_width+15, game_depth+2);
	die(LE_SCREEN, fb);
    };

    if (!strcmp(ttytype, "dumb"))
	die(LE_SCREEN, "your window does not have terminal emulation facilities.");

    draw_frame();
    print_msg ("Press a key:");
    flush_keyboard();
    (void)get_key();

    while (1) {
	char ch;

	game_loop();
	print_msg ("Play again?");
	flush_keyboard ();
	ch = get_key();
	while (ch!='y' && ch!='Y' && ch!='n' && ch !='N')
	    ch = get_key();

	if (ch=='n' || ch=='N') return;
    };
}

setup_curses()
{
    initscr();
    clear();
    noecho();
    cbreak();
    in_curses = 1;
}

force_refresh()
{
    clear();
    draw_frame();
    draw_score();
    redraw_board();
}

clear_board()
{
    register int ix;

    for (ix=0; ix<(game_width * game_depth); ix++)
	game_board[ix] = 0;
}

char getkeystroke(total_time) /* returns \0 for timeout */
long total_time;
{
    fd_set read_fds;		/* Select must look at stdin */
    struct timeval timeout;	/* Time before piece drops in select(2) */
    struct timeval *tvp;
    char ch;			/* Keystroke (command) */
    int nfound;

    /* FD_ZERO (&read_fds);   do these the hard way;
    FD_SET (0, &read_fds); */
    bzero((char *)(&read_fds), sizeof(read_fds));
    read_fds.fds_bits[0] |= (1 << 0);
    if (total_time!=0) {
	timeout.tv_sec = 0;
	timeout.tv_usec = total_time;
	tvp = &timeout;
    }
    else {
	tvp = (struct timeval *)NULL;
    };
    nfound = select (1, &read_fds, (fd_set*) NULL, (fd_set*) NULL, tvp);
    if (nfound == 0 || (nfound == (-1) && errno == EINTR)) {
	return 0;
    }
    else {
	(void)read (0, &ch, 1);
	return ch;
    };
}

int collide(ptr, px, py) /* returns 0 for ok, 1 for collision, 2 for ok-but-offscreen */
gpiece *ptr;
int px, py;
{
    register int ix, iy;
    int posx, posy;
    char outflag = 1;

    for (iy=ptr->dep-1; iy>=0; iy--)
	for (ix=ptr->wid-1; ix>=0; ix--) 
	    if (ptr->blk[iy*(ptr->wid)+ix]) {
		posx = px-(ptr->cenx)+ix;
		posy = py-(ptr->ceny)+iy;
		if (outflag && (posy>=0)) outflag = 0;
		if (posy >= game_depth || posx < 0 || posx >= game_width ||
		    (posy>=0 && game_board[posy*game_width+posx]))
		    return 1;
	    }
    if (outflag) return 2;
    else return 0;
}

emplace_piece(ptr, px, py)
gpiece *ptr;
int px, py;
{
    int posx, posy; /* coords of top corner */
    register int ix, iy;

    posx = px-(ptr->cenx);
    posy = py-(ptr->ceny);

    for (iy=ptr->dep-1; iy>=0; iy--)
	for (ix=ptr->wid-1; ix>=0; ix--) 
	    if (ptr->blk[iy*(ptr->wid)+ix]) 
		if (posy+iy>=0) {
		    game_board[(posy+iy)*game_width+(posx+ix)] = 1;
		}

}

smush_board()
{
    register int ix, iy;
    int solid;
    int displace;
    char tc;

    displace=0;
    for (iy = game_depth-1; iy>=0; iy--) {
	solid = 1;
	for (ix=0; (ix<game_width); ix++) {
	    tc = game_board[iy*game_width+ix];
	    if (displace!=0)
		game_board[(iy+displace)*game_width+ix] = tc;
	    if (!tc) 
		solid = 0;
	};
	if (solid) 
	    displace++;
    };

    if (displace>0) {
	for (iy=0; iy<displace; iy++)
	    for (ix=0; (ix<game_width); ix++) 
		game_board[iy*game_width+ix] = 0;
	redraw_board();
    };
    score += displace*(9+displace);
}

game_loop() /* plays a single game */
{
    char ch, stickflag;
    int px, py, pnum, rot, nrot;
    gpiece *curptr, *ncurptr;
    int dropcycle;

    clear_board();
    redraw_board();
    score=0;
    game_speed = 40000;
    move(7, 2*game_width+6);
    clrtoeol();
    draw_score();
    print_msg("");
    myrefresh();

    while (1) {
	pnum = random() % num_pieces;
	rot = 0;
	curptr = &(piece_list[pnum][rot]);
	px = game_width / 2;
	py = curptr->ceny - curptr->dep;
	if (collide(curptr, px, py) == 1) {
	    print_msg("unable to place first piece");
	    return;
	};
	
	while ((ch = collide(curptr, px, py+1)) == 2) py++;
	if (ch == 1) {
	    /* game over */
	    print_msg("Game Over");
	    return;
	};

	draw_piece(curptr, px, py, '#');
	myrefresh();

	stickflag = 0;
	dropcycle = 0;
	while (!stickflag) {
	    draw_piece(curptr, px, py, '\0');

	    ch = getkeystroke(game_speed);
	    if (ch=='s') {
		ch = getkeystroke((long)0);
		ch = getkeystroke(game_speed);
	    };
	    switch (ch) {
		case '\0':
		    break;
		case '8':
		case '2':
		case 'i':
		case 'm':
		    dropcycle = 999;
		    break;
		case ' ':
		    while (collide(curptr, px, py+1) == 0) {
			score++;
			py++;
		    };
		    stickflag=1;
		    break;
		case '4':
		case 'j':
		    if (collide(curptr, px-1, py) == 0)
			px--;
		    break;
		case '5':
		case 'k':
		    nrot = (rot+3) % 4;
		    ncurptr = &(piece_list[pnum][nrot]);
		    if (collide(ncurptr, px, py) == 0) {
			rot = nrot;
			curptr = ncurptr;
		    };
		    break;
		case '6':
		case 'l':
		    if (collide(curptr, px+1, py) == 0)
			px++;
		    break;
		case '\014':
		    force_refresh();
		    break;
		case 'q':
		    print_msg("Quitting");
		    return;
	    };
	    if (!stickflag) {
		dropcycle++;
		if (dropcycle > 9) {
		    dropcycle = 0;
		    if (collide(curptr, px, py+1) == 0)
			py++;
		    else
			stickflag=1;
		};
	    };	

	    draw_piece(curptr, px, py, '#');
	    myrefresh();
	}; 
	/* piece just got stuck */
	score++;
	if (game_speed > 8000)
	    game_speed -= 100;
	draw_piece(curptr, px, py, ' ');
	emplace_piece(curptr, px, py);
	smush_board();
	draw_score();
    }
}

