/* ntris -- a product of Sadistic Software. Copyright 1990, Andrew Plotkin. */

#include <stdio.h>
#include "ntris.h"

extern char *malloc();

load_pieces(flnm)
char *flnm;
{
    FILE *infl;
    int piecex, piecey, cenx, ceny;
    char cenxs, cenys;
    int ix, jx, tx, ty;
    char tc;

    infl = fopen(flnm, "r");
    if (infl == (FILE *)NULL) {
	perror("ntris");
	die(LE_OPEN, "unable to open piece file");
    };
    
    if (fscanf(infl, " %d, %d\n", &game_width, &game_depth)==0)
	die(LE_PIECE, "error in piece file");
    if (game_width<0 || game_depth<0)
	die(LE_PIECE, "illegal game board size");
    game_board = malloc((unsigned)(game_width * game_depth));
    if (game_board == NULL)
	die(LE_MALLOC, "error in malloc()");

    if (fscanf(infl, " %d\n", &num_pieces)==0)
	die(LE_PIECE, "error in piece file");
    if (num_pieces<0 || num_pieces>=MAX_PIECES)
	die(LE_PIECE, "illegal number of pieces");

    for (ix=0; ix<num_pieces; ix++) {
	if (feof(infl)) 
	    die(LE_PIECE, "unexpected end of piece file");
	if (fscanf(infl, " %d, %d, %d %c, %d %c\n", &piecex, &piecey, &cenx, &cenxs, &ceny, &cenys)==0)
	    die(LE_PIECE, "error in piece file");
	if (cenxs==',' || cenys==',')
	    die(LE_PIECE, "error in piece file");
	if (cenxs=='-') cenxs='-';
	else cenxs='+';
	if (cenys=='-') cenys='-';
	else cenys='+';
	for (jx=0; jx<4; jx++) {
	    piece_list[ix][jx].blk = malloc((unsigned)(piecex*piecey));
	    if (piece_list[ix][jx].blk==NULL)
		die(LE_MALLOC, "error in malloc()!");
	};

	for (ty=0; ty<piecey; ty++) {
	    for (tx=0; tx<piecex; tx++) {
		char tc = getc(infl);
		if (tc=='#')
		    piece_list[ix][0].blk[ty*piecex+tx] = 1;
		else
		    piece_list[ix][0].blk[ty*piecex+tx] = 0;
	    };
	    tc = getc(infl);
	    while (!feof(infl) && tc!='\n') {
		tc = getc(infl);
	    };
	};
	fourk_piece(ix, piecex, piecey, cenx-1, cenxs, ceny-1, cenys);
    };
}

fourk_piece(pnum, width, depth, cenx, cenxs, ceny, cenys)
int pnum, width, depth;
int cenx;
char cenxs;
int ceny;
char cenys;
{
    register int ix, iy;
    char tc;

    piece_list[pnum][0].wid = width;
    piece_list[pnum][0].dep = depth;
    piece_list[pnum][1].wid = depth;
    piece_list[pnum][1].dep = width;
    piece_list[pnum][2].wid = width;
    piece_list[pnum][2].dep = depth;
    piece_list[pnum][3].wid = depth;
    piece_list[pnum][3].dep = width;

    for (iy=0; iy<depth; iy++)
	for (ix=0; ix<width; ix++) {
	    tc = piece_list[pnum][0].blk[(iy)*width+(ix)];
	    piece_list[pnum][1].blk[(ix)*depth+(depth-1-iy)] = tc;
	    piece_list[pnum][2].blk[(depth-1-iy)*width+(width-1-ix)] = tc;
	    piece_list[pnum][3].blk[(width-1-ix)*depth+(iy)] = tc;
	}

    piece_list[pnum][0].cenx = cenx;
    piece_list[pnum][0].ceny = ceny;
    piece_list[pnum][1].cenx = (depth-1-ceny) + (cenys=='-' ? 1 : 0);
    piece_list[pnum][1].ceny = cenx;
    piece_list[pnum][2].cenx = (width-1-cenx) + (cenxs=='-' ? 1 : 0);
    piece_list[pnum][2].ceny = (depth-1-ceny) + (cenys=='-' ? 1 : 0);
    piece_list[pnum][3].cenx = ceny;
    piece_list[pnum][3].ceny = (width-1-cenx) + (cenxs=='-' ? 1 : 0);
}

dump_piece(pc)
gpiece *pc;
{
    register int ix, iy;

    printf("piece is %d by %d (%d,%d)\n", pc->wid, pc->dep, pc->cenx, pc->ceny);
    for (iy=0; iy<pc->dep; iy++) {
	for (ix=0; ix<pc->wid; ix++)
	    if (ix==pc->cenx && iy==pc->ceny)
		(void)putchar(pc->blk[iy*(pc->wid)+ix] ? '@' : 'O');
	    else
		(void)putchar(pc->blk[iy*(pc->wid)+ix] ? '#' : '.');
	(void)putchar('\n');
    };
}

dump_set(pnum, cenflag)
int pnum;
char cenflag;
{
    register int ix, iy, rot;
    int winwid;
    gpiece *pc;

#define MAX(x, y) ((x>y) ? x : y)

    winwid = MAX(piece_list[pnum][0].wid, piece_list[pnum][0].dep);

    for (iy=0; iy<winwid; iy++) {
	for (rot=0; rot<4; rot++) {
	    for (ix=0; ix<winwid; ix++) {
		if (ix < piece_list[pnum][rot].wid &&
		    iy < piece_list[pnum][rot].dep) {
		    pc = &piece_list[pnum][rot];
		    if (cenflag && ix==pc->cenx && iy==pc->ceny)
			(void)putchar(pc->blk[iy*(pc->wid)+ix] ? '@' : 'O');
		    else
			(void)putchar(pc->blk[iy*(pc->wid)+ix] ? '#' : '.');
		}
		else {
		    (void)putchar(' ');
		}
	    }
	    (void)putchar(' ');
	    (void)putchar(' ');
	};
	(void)putchar('\n');
    };
    (void)putchar('\n');
}
